package module.engine.md25;

import java.nio.ByteBuffer;
import java.util.HashMap;

import exception.ArgumentMismatchException;
import exception.MissingCommandException;

public class MD25EngineTranslator {
	
	/*|  CODE  |      NAME		|CMDSIZE|RETSIZE|							Description	    					 |
		0x21	GET SPEED 1 		2		1	 returns the current requested speed of motor 1
		0x22	GET SPEED 2			2		1	 returns the current requested speed of motor 2
		0x23	GET ENCODER 1		2		4	 motor 1 encoder count, 4 bytes returned high byte first (signed)
		0x24	GET ENCODER 2		2		4	 motor 2 encoder count, 4 bytes returned high byte first (signed)
		0x25	GET ENCODERS		2		8	 returns 8 bytes -  encoder1 count, encoder2 count
		0x26	GET VOLTS			2		1	 returns the input battery voltage level
		0x27	GET CURRENT 1		2		1	 returns the current drawn by motor 1
		0x28 	GET CURRENT 2		2		1	 returns the current drawn by motor 1
		0x29	GET VERSION			2		1	 returns the MD25 software version
		0x2A	GET ACCELERATION	2		1	 returns the current acceleration level
		0x2B	GET MODE			2		1	 returns the currently selected mode
		0x2C	GET VI				2		3	 returns battery volts, motor1 current and then motor2 current 
		0x31	SET SPEED 1			3		0	 set new speed1
		0x32	SET SPEED 2/TURN	3		0	 set new speed2 or turn
		0x33	SET ACCELERATION	3		0	 set new acceleration
		0x34	SET MODE			3		0	 set the mode
		0x35	RESET ENCODERS		2		0	 zero both of the encoder counts
		0x36	DISABLE REGULATOR	2		0	 power output not changed by encoder feedback
		0x37	ENABLE REGULATOR	2		0	 power output is regulated by encoder feedback
		0x38	DISABLE TIMEOUT		2		0	 MD25 will continuously output with no regular commands
		0x39	ENABLE TIMEOUT		2		0	 MD25 output will stop after 2 seconds without communication		*/
	
	private static final int CODE = 0;
	private static final int SIZE = 1;
	private static final int RTSZ = 2;
	private static HashMap<MD25EngineInstruction,int[]> cmds;
	
	static {
		cmds = new HashMap<MD25EngineInstruction,int[]>(MD25EngineInstruction.values().length);
		int[] gs1 = {0x21,2,1,0};
		cmds.put(MD25EngineInstruction.GETSPEED1,gs1);
		int[] gs2 = {0x22,2,1,0};
		cmds.put(MD25EngineInstruction.GETSPEED2,gs2);
		int[] ge1 = {0x23,2,4,0};
		cmds.put(MD25EngineInstruction.GETENCONDER1,ge1);
		int[] ge2 = {0x24,2,4,0};
		cmds.put(MD25EngineInstruction.GETENCONDER2,ge2);
		int[] ges = {0x25,2,8,0};
		cmds.put(MD25EngineInstruction.GETENCODERS,ges);
		int[] gvo = {0x26,2,1,0};
		cmds.put(MD25EngineInstruction.GETVOLTS,gvo);
		int[] gc1 = {0x27,2,1,0};
		cmds.put(MD25EngineInstruction.GETCURRENT1,gc1);
		int[] gc2 = {0x28,2,1,0};
		cmds.put(MD25EngineInstruction.GETCURRENT2,gc2);
		int[] gve = {0x29,2,1,0};
		cmds.put(MD25EngineInstruction.GETVERSION,gve);
		int[] ga = {0x2A,2,1,0};
		cmds.put(MD25EngineInstruction.GETACCELERATION,ga);
		int[] gm = {0x2B,2,1,0};
		cmds.put(MD25EngineInstruction.GETMODE,gm);
		int[] gvi = {0x2C,2,3,0};
		cmds.put(MD25EngineInstruction.GETVI,gvi);
		int[] ss1 = {0x31,3,0,0};
		cmds.put(MD25EngineInstruction.SETSPEED1,ss1);
		int[] ss2 = {0x32,3,0,0};
		cmds.put(MD25EngineInstruction.SETSPEED2,ss2);
		int[] sa = {0x33,3,0,0};
		cmds.put(MD25EngineInstruction.SETACCELERATION,sa);
		int[] sm = {0x34,3,0,0};
		cmds.put(MD25EngineInstruction.SETMODE,sm);
		int[] res = {0x35,2,0,0};
		cmds.put(MD25EngineInstruction.RESETENCODERS,res);
		int[] dr = {0x36,2,0,0};
		cmds.put(MD25EngineInstruction.DISABLEREGULATOR,dr);
		int[] er = {0x37,2,0,0};
		cmds.put(MD25EngineInstruction.ENABLEREGULATOR,er);
		int[] dt = {0x38,2,0,0};
		cmds.put(MD25EngineInstruction.DISABLETIMEOUT,dt);
		int[] et = {0x39,2,0,0};
		cmds.put(MD25EngineInstruction.ENABLETIMEOUT,et);	
	}
	
	public static final byte STARTBYTE = 0x00;
	
	public static int getCode(MD25EngineInstruction c) throws MissingCommandException {
		if(cmds.containsKey(c))
			return cmds.get(c)[CODE];
		throw new MissingCommandException(c.toString());
	}

	public static int getSize(MD25EngineInstruction c) throws MissingCommandException {
		if(cmds.containsKey(c))
			return cmds.get(c)[SIZE];
		throw new MissingCommandException(c.toString());
	}

	public static int getRetSize(MD25EngineInstruction c) throws MissingCommandException {
		if(cmds.containsKey(c))
			return cmds.get(c)[RTSZ];
		throw new MissingCommandException(c.toString());
	}
	
	public static boolean isQuery(MD25EngineInstruction c) throws MissingCommandException {
		if(cmds.containsKey(c))
			return cmds.get(c)[RTSZ] > 0;
		throw new MissingCommandException(c.toString());
	}
	
	public static byte[] makeCmd(MD25EngineInstruction c,byte[] args) throws ArgumentMismatchException, MissingCommandException {
		int cmdSz=getSize(c);
		int argSz=cmdSz-2; //start + command bytes = 2
		if(args == null && argSz != 0)
			throw new ArgumentMismatchException("expected argument size - " + argSz + ", provided argument size - 0");
		if(args != null && args.length!=argSz)
			throw new ArgumentMismatchException("expected argument size - " + argSz + ", provided argument size - " + args.length);
		byte[] cmd = new byte[cmdSz];
		cmd[0]=STARTBYTE;
		cmd[1]=(byte) getCode(c);
		for(int i =0; i < argSz;i++)
			cmd[i+2]=args[i];
		return cmd;
	}
	
	public static double answer2double(MD25EngineInstruction i, byte[] b) {
		switch(i) {
		case GETVOLTS:
		case GETCURRENT1:
		case GETCURRENT2:
			return b[0]/10;
		case GETSPEED1:
		case GETSPEED2:
		case GETMODE:
		case GETVERSION:
		case GETACCELERATION:
			return b[0];
		default: // all others must reinterpret the double they receive as bytes :(
			return answer2double(b);	
		}
	}
	
	public static double answer2double(byte[] b) {
		if(b.length < 8) {
			byte[] temp = new byte[8];
			int i = 0;
			for(; i < b.length; i++)
				temp[i] = b[i];
			for(; i < 8; i++)
				temp[i] = 0;
			b = temp;
		}
		ByteBuffer buf = ByteBuffer.wrap(b);
		return Double.longBitsToDouble(buf.getLong());		
	}
}
