package module.vision.webcam;

import java.util.ArrayList;
import java.util.Map;
import module.vision.CentroidMap;
import module.vision.Image;
import module.vision.Vision;
import util.VisibilityOptions;
import util.VisionOptions;
import util.WebCameraUtil;
import com.googlecode.javacv.*;
import com.googlecode.javacv.cpp.opencv_core.CvPoint;
import com.googlecode.javacv.cpp.opencv_core.CvScalar;
import com.googlecode.javacv.cpp.opencv_core.IplImage;
import dto.Synchronized;
import exception.VisionFailureException;
import static com.googlecode.javacv.cpp.opencv_core.*;
import static com.googlecode.javacv.cpp.opencv_imgproc.*;

public class WebCameraVision extends Vision {
	private CanvasFrame vision_window;
	private CanvasFrame orientation_window;
	private OpenCVFrameGrabber camera;
	private int threshold 			= VisionOptions.threshold;
	private int max_value			= VisionOptions.imgMaxValue;
	private int max_contour_area	= VisionOptions.MaxArea;
	
	public WebCameraVision(Synchronized<dto.vision.VisionInputDTO> input,Synchronized<dto.vision.VisionOutputDTO> output) {
		super(input, output);
	}
	
	@Override
	protected void start() throws Exception {		
		if(VisibilityOptions.show_windows) {
			vision_window		= new CanvasFrame("Vision Window");
			orientation_window = new CanvasFrame("Orientation Window");
		}
		camera = WebCameraUtil.captureCamera(VisionOptions.camID,VisionOptions.camWidth,VisionOptions.camHeight);
		//camera.start();
		on=true;
	}
	
	@Override
	protected void stop() {
		if(VisibilityOptions.show_windows) {
			vision_window.dispose();
			orientation_window.dispose();
		}
		on=false;
	}

	@Override
	protected Image<?> perceiveVision() throws VisionFailureException {
	    IplImage frame = null;
	    
	    try {
			frame = camera.grab();
		} catch (Exception e) {
			throw new VisionFailureException(e);
		}
	    
	    return new Image<IplImage>(frame);
	}

	@Override
	protected CentroidMap<?> processVision(Image<?> gen_image) {
		IplImage image = (IplImage) gen_image.image;
		
    	CvSize	image_size	= cvSize(image.width(), image.height());
		
	    IplImage grayImg	= cvCreateImage(image_size, IPL_DEPTH_8U, 1);
	    IplImage bwImg		= cvCreateImage(image_size, IPL_DEPTH_8U, 1);
				
    	/**
    	 * Get a gray-scale image from the frame.
    	 */
		cvCvtColor(image, grayImg, CV_RGB2GRAY);
				
		/**
		 * Get a black and white image from the gray-scale image.
		 */
		cvThreshold(grayImg, bwImg, threshold, max_value, CV_THRESH_BINARY_INV);
		
		Map<CvPoint,Integer> centroids = WebCameraUtil.computeCentroids(bwImg, max_contour_area);
		
		if(VisibilityOptions.show_windows) {
						
			ArrayList<CvPoint> keys = new ArrayList<CvPoint>(centroids.keySet());
			
			if(!keys.isEmpty()) {
				CvPoint bigger = keys.get(0);
				if(VisibilityOptions.draw_all_centroids){
					for(int i = 1; i < keys.size(); i++) {
						CvPoint curr = keys.get(i);
						/**
						 * Draw a circle for each centroid.
						 */
						if(centroids.get(bigger) > centroids.get(curr)) {
							cvCircle(bwImg, curr, 5, CvScalar.WHITE, 2, 8, 0);
						} else {
							cvCircle(bwImg, bigger, 5, CvScalar.WHITE, 2, 8, 0);
							bigger = curr;
						}
					}
				} else {
					for(int i = 1; i < keys.size(); i++) {
						CvPoint curr = keys.get(i);
						if(centroids.get(bigger) < centroids.get(curr))
							bigger = curr;
					}
				}
				/**
				 * Draw a thicker circle, for the centroid with bigger area (map's last element)
				 */
				cvCircle(bwImg, bigger, 5, CvScalar.WHITE, 5, 8, 0);
				
				vision_window.showImage(image);
				orientation_window.showImage(bwImg);
			}
			
		}

		return new CentroidMap<CvPoint>(centroids);
	}

	@Override
	protected void act() {
			threshold = instructions.grayscale_threshold;
			max_value = instructions.grayscale_max_value;
			max_contour_area = instructions.contour_max_area;		
	}
}
